"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = pureFuncCache;

var _store = _interopRequireDefault(require("./store"));

var _assist = require("../utils/assist");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 纯函数执行的结果缓存在内存中，下次调用直接使用缓存数据
 * @param {Function} fn - 返回值是Promise的函数
 * @returns {Function} 和原函数的调用方式是一样的，第一次调用数据缓存后，之后的调用不会调用原函数，而是会直接使用缓存后的数据，另外该函数还带有一个静态的clearCache方法，可以用来清空缓存数据
 * @property {Function} clearCache 清空缓存数据
 */
function pureFuncCache(fn) {
  var cache = {};

  var cachedFn = function cachedFn() {
    var cacheKey = _assist.argToKey.apply(void 0, arguments);

    return cache.hasOwnProperty(cacheKey) ? cache[cacheKey] : cache[cacheKey] = fn.apply(void 0, arguments);
  };

  cachedFn.clearCache = function (key) {
    if (key) {
      delete cache[key];
    } else {
      cache = {};
    }
  };

  cachedFn.funType = 'pureFunc';
  cachedFn.cacheType = 'memory';
  cachedFn.originFunName = fn.name;

  _store.default.push(cachedFn);

  return cachedFn;
}